package me.km.permissions;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import me.km.Server;
import me.km.events.CommandEvent;
import me.km.snuviscript.ScriptEvents;
import me.km.snuviscript.Scripts;
import net.minecraft.command.CommandSource;
import net.minecraft.command.Commands;
import net.minecraft.command.ICommandSource;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.PlayerEntity;

public class ModCommandManager extends Commands
{
    private final HashSet<String> vanilla = new HashSet<>();
    private final HashMap<String, Command> commands = new HashMap<>();
    private final PermissionManager perms;
    private final ScriptEvents events;
    private final Scripts scripts;

    public ModCommandManager(boolean isDedicatedServer, PermissionManager perms, ScriptEvents events, Scripts scripts)
    {
        super(isDedicatedServer);
        getDispatcher().getRoot().getChildren().forEach(c -> vanilla.add(c.getName()));
        this.perms = perms;
        this.events = events;
        this.scripts = scripts;
    }
    
    public void registerCommand(Command command)
    {
        commands.put(command.getName(), command);
        for(String alias : command.getAliases())
        {
            commands.put(alias, command);
        }
    }

    private String getCommandName(String rawCommand)
    {
        if(rawCommand.isEmpty())
        {
            return "";
        }
        int index = rawCommand.indexOf(' ');
        return rawCommand.substring(rawCommand.charAt(0) == '/' ? 1 : 0, index == -1 ? rawCommand.length() : index);
    }
    
    private static String[] getArguments(String rawCommand)
    {
        int old = rawCommand.indexOf(' ') + 1;
        if(old == 0)
        {
            return new String[0];
        }
        int pos = old;
        ArrayList<String> list = new ArrayList<>();
        while(pos < rawCommand.length())
        {
            char c = rawCommand.charAt(pos);
            switch(c)
            {
                case ' ':
                    if(pos - old > 0)
                    {
                        list.add(rawCommand.substring(old, pos));
                    }
                    old = pos + 1;
                    break;
                case '"':
                    if(pos - old > 0)
                    {
                        list.add(rawCommand.substring(old, pos));
                    }
                    old = pos + 1;
                    pos = old;
                    while(pos < rawCommand.length() && rawCommand.charAt(pos) != '"')
                    {
                        pos++;
                    }
                    list.add(rawCommand.substring(old, pos));
                    old = pos + 1;
                    break;
            }
            pos++;
        }
        if(pos - old > 0)
        {
            list.add(rawCommand.substring(old, pos));
        }
        return list.toArray(new String[list.size()]);
    }

    private ICommandSource getSource(CommandSource cs)
    {
        if(cs.getEntity() != null)
        {
            return cs.getEntity();
        }
        return cs.getServer();
    }
    
    @Override
    public int handleCommand(CommandSource cs, String rawCommand)
    {
        String commandName = getCommandName(rawCommand);
        
        Command command = commands.get(commandName);
        if(command != null)
        {
            if(perms.hasPermission(cs, commandName))
            {
                command.execute(getSource(cs), getArguments(rawCommand));
                return 1;
            }
            events.onMissingPermission(getSource(cs), commandName);
            return 0;
        }
        
        if(scripts.isRegisteredScriptCommand(commandName))
        {
            Entity ent = cs.getEntity();
            if(ent != null && ent instanceof PlayerEntity)
            {
                events.onCustomCommand((PlayerEntity) ent, commandName, getArguments(rawCommand));
                return 1;
            }
            return 0;
        }
        
        if(vanilla.contains(commandName))
        {
            if(perms.hasPermission(cs, commandName))
            {
                Entity ent = cs.getEntity();
                if(ent != null && ent instanceof PlayerEntity)
                {
                    CommandEvent e = new CommandEvent((PlayerEntity) ent, commandName);
                    events.onCommand(e);
                    if(e.isCanceled())
                    {
                        return 0;
                    }
                }
                return super.handleCommand(cs, rawCommand);
            }
            events.onMissingPermission(getSource(cs), commandName);
            return 0;
        }
        
        events.onMissingCommand(getSource(cs), commandName);
        return 0;
    }

    //@Override
    //public void sendCommandListPacket(EntityPlayerMP player)
    //{
    //    super.sendCommandListPacket(player);
    //    LiteralCommandNode<ISuggestionProvider> test = new LiteralCommandNode<>(
    //            "wusi", null, (c) -> true, null, null, false);
    //    ArgumentCommandNode<ISuggestionProvider, String> a = 
    //            new ArgumentCommandNode<>("ok", StringArgumentType.string(), null,
    //        (c) -> true, null, null, false, null);
    //    a.addChild(new ArgumentCommandNode<>("ok 2", StringArgumentType.string(), null,
    //        (c) -> false, null, null, false, null));
    //    test.addChild(a);
    //    rootcommandnode.addChild(test);
    //}
}
